@@ -19,6 +19,7 @@
 /**********************************************
  * matrix.c : matrix and other 3D math functions
  * Copyright (C) 2001-2003 Bertrand 'blam' LAMY
+ * Copyright (C) 2003-2006 Jean-Baptiste 'Jiba' LAMY
  **********************************************/
 
 #include <stdlib.h>
@@ -524,6 +525,7 @@ void matrix_from_quaternion (GLfloat m[19], GLfloat q[4]) {
   m[18] = 1.0;
 }
 
+/* Old bug'ed version
 void quaternion_from_matrix (GLfloat q[4], GLfloat m[16]) {
   GLfloat s;
   s = sqrt (fabs (m[0] + m[5] + m[10] + m[15]));
@@ -561,6 +563,38 @@ void quaternion_from_matrix (GLfloat q[4], GLfloat m[16]) {
   }
   quaternion_normalize (q);
 }
+*/
+
+void quaternion_from_matrix (GLfloat q[4], GLfloat m[16]) {
+  GLfloat s;
+  s = 1 + m[0] + m[5] + m[10];
+  if (s > 0.00000001) {
+    s    = sqrt(fabs(s)) * 2;
+    q[0] = -(m[9] - m[6]) / s;
+    q[1] = -(m[2] - m[8]) / s;
+    q[2] = -(m[4] - m[1]) / s;
+    q[3] = 0.25 * s;
+  } else if ((m[0] > m[5]) && (m[0] > m[10])) {
+    s    = sqrt(fabs(1.0 + m[0] - m[5] - m[10])) * 2;
+    q[0] = 0.25 * s;
+    q[1] = -(m[4] + m[1]) / s;
+    q[2] = -(m[2] + m[8]) / s;
+    q[3] = -(m[9] - m[6]) / s;
+  } else if (m[5] > m[10]) {
+    s    = sqrt(fabs(1.0 + m[5] - m[0] - m[10])) * 2;
+    q[0] = -(m[4] + m[1]) / s;
+    q[1] = 0.25 * s;
+    q[2] = -(m[9] + m[6]) / s;
+    q[3] = -(m[2] - m[8]) / s;
+  } else {
+    s    = sqrt(fabs(1.0 + m[10] - m[0] - m[5])) * 2;
+    q[0] = -(m[2] + m[8]) / s;
+    q[1] = -(m[9] + m[6]) / s;
+    q[2] = 0.25 * s;
+    q[3] = -(m[4] - m[1]) / s;
+  }
+  quaternion_normalize(q);
+}
 
 void quaternion_rotation (GLfloat q[4], GLfloat v[3], GLfloat angle) {
   GLfloat s;
@@ -704,6 +738,18 @@ void point_by_matrix_copy (GLfloat r[3], GLfloat p[3], GLfloat m[19]) {
   r[2] = p[0] * m[2] + p[1] * m[6] + p[2] * m[10] + m[14];
 }
 
+void sphere_by_matrix_copy (GLfloat r[4], GLfloat p[4], GLfloat m[19]) {
+  GLfloat scaling;
+  scaling = m[16];
+  if (m[17] > scaling) { scaling = m[17]; }
+  if (m[18] > scaling) { scaling = m[18]; }
+  
+  r[0] = p[0] * m[0] + p[1] * m[4] + p[2] * m[ 8] + m[12];
+  r[1] = p[0] * m[1] + p[1] * m[5] + p[2] * m[ 9] + m[13];
+  r[2] = p[0] * m[2] + p[1] * m[6] + p[2] * m[10] + m[14];
+  r[3] = p[3] * scaling;
+}
+
 void point4_by_matrix (GLfloat p[4], GLfloat m[19]) {
   GLfloat x; GLfloat y; GLfloat z; GLfloat w;
   x = p[0] * m[0] + p[1] * m[4] + p[2] * m[ 8] + p[3] * m[12];
@@ -716,7 +762,7 @@ void point4_by_matrix (GLfloat p[4], GLfloat m[19]) {
   p[3] = w;
 }
 
-void point_rotate_lateral (GLfloat p[3], GLfloat angle) {
+void point_rotate_y (GLfloat p[3], GLfloat angle) {
   GLfloat old_1;
   GLfloat old_2;
   GLfloat co = cos (angle);
@@ -728,7 +774,7 @@ void point_rotate_lateral (GLfloat p[3], GLfloat angle) {
   p[2] = old_2 * co + old_1 * si;
 }
 
-void point_rotate_vertical (GLfloat p[3], GLfloat angle) {
+void point_rotate_x (GLfloat p[3], GLfloat angle) {
   GLfloat old_1;
   GLfloat old_2;
   GLfloat co = cos (angle);
@@ -740,7 +786,7 @@ void point_rotate_vertical (GLfloat p[3], GLfloat angle) {
   p[2] = old_2 * co + old_1 * si;
 }
 
-void point_rotate_incline (GLfloat p[3], GLfloat angle) {
+void point_rotate_z (GLfloat p[3], GLfloat angle) {
   GLfloat old_1;
   GLfloat old_2;
   GLfloat co = cos (angle);
@@ -852,13 +898,13 @@ void matrix_scale(GLfloat m[19], GLfloat x, GLfloat y, GLfloat z) {
   m[18] *= z;
 }
 
-void matrix_turn_lateral (GLfloat m[19], GLfloat angle) {
+void matrix_turn_y (GLfloat m[19], GLfloat angle) {
   GLfloat c; GLfloat s;
   GLfloat m0; GLfloat m1; GLfloat m2; 
   /* rotate along Y axis of the matrix */
   /* Following code is equivalent (with simplifications!) to:
    *   matrix_set_identity (b);
-   *   matrix_rotate_lateral (b, angle);
+   *   matrix_rotate_y (b, angle);
    *   multiply_matrix (r, m, b);
    */
   c = cos (angle);
@@ -874,13 +920,13 @@ void matrix_turn_lateral (GLfloat m[19], GLfloat angle) {
   m[10] = s * m2 + c * m[10];
 }
 
-void matrix_turn_vertical (GLfloat m[19], GLfloat angle) {
+void matrix_turn_x (GLfloat m[19], GLfloat angle) {
   GLfloat c; GLfloat s;
   GLfloat m4; GLfloat m5; GLfloat m6; 
   /* rotate along X axis of the matrix */
   /* Following code is equivalent (with simplifications!) to:
    *   matrix_set_identity (b);
-   *   matrix_rotate_vertical (b, angle);
+   *   matrix_rotate_x (b, angle);
    *   multiply_matrix (r, m, b);
    */
   c = cos (angle);
@@ -896,13 +942,13 @@ void matrix_turn_vertical (GLfloat m[19], GLfloat angle) {
   m[10] = - s * m6 + c * m[10];
 }
 
-void matrix_turn_incline (GLfloat m[19], GLfloat angle) {
+void matrix_turn_z (GLfloat m[19], GLfloat angle) {
   GLfloat c; GLfloat s;
   GLfloat m0; GLfloat m1; GLfloat m2;
   /* rotate along Z axis of the matrix */
   /* Following code is equivalent (with simplifications!) to:
    *   matrix_set_identity (b);
-   *   matrix_rotate_incline (b, angle);
+   *   matrix_rotate_z (b, angle);
    *   multiply_matrix (r, m, b);
    */
   c = cos (angle);
@@ -1012,7 +1058,7 @@ void matrix_rotate_axe (GLfloat m[19], GLfloat angle, GLfloat x, GLfloat y, GLfl
   m[10] = ox * f7 + oy * f8 + oz * f9;
 }
 
-void matrix_rotate_lateral (GLfloat m[19], GLfloat angle) {
+void matrix_rotate_y (GLfloat m[19], GLfloat angle) {
   GLfloat c; GLfloat s;
   GLfloat m0; GLfloat m4; GLfloat m8;
   /* rotate along Y axis of parent with matrix origin */
@@ -1033,7 +1079,7 @@ void matrix_rotate_lateral (GLfloat m[19], GLfloat angle) {
    */
 }
 
-void matrix_rotate_vertical (GLfloat m[19], GLfloat angle) {
+void matrix_rotate_x (GLfloat m[19], GLfloat angle) {
   GLfloat c; GLfloat s;
   GLfloat m1; GLfloat m5; GLfloat m9;
   /* rotate along X axis of parent with matrix origin */
@@ -1054,7 +1100,7 @@ void matrix_rotate_vertical (GLfloat m[19], GLfloat angle) {
    */
 }
 
-void matrix_rotate_incline (GLfloat m[19], GLfloat angle) {
+void matrix_rotate_z (GLfloat m[19], GLfloat angle) {
   GLfloat c; GLfloat s; 
   GLfloat m0; GLfloat m4; GLfloat m8;
   /* rotate along Z axis of parent with matrix origin */
@@ -1986,18 +2032,12 @@ Frustum* frustum_by_matrix (Frustum* r, Frustum* f, GLfloat* m) {
   if (m[17] > scaling) { scaling = m[17]; }
   if (m[18] > scaling) { scaling = m[18]; }
   /* re-compute the normals */
-  face_normal (r->planes, r->points, r->points + 3, r->points + 9);
-  vector_set_length (r->planes, scaling);
-  face_normal (r->planes + 4, r->points + 12, r->points + 15, r->points);
-  vector_set_length (r->planes + 4, scaling);
-  face_normal (r->planes + 8, r->points + 9, r->points + 6, r->points + 21);
-  vector_set_length (r->planes + 8, scaling);
-  face_normal (r->planes + 12, r->points + 12, r->points, r->points + 21);
-  vector_set_length (r->planes + 12, scaling);
-  face_normal (r->planes + 16, r->points + 3, r->points + 15, r->points + 6);
-  vector_set_length (r->planes + 16, scaling);
-  face_normal (r->planes + 20, r->points + 15, r->points + 12, r->points + 18);
-  vector_set_length (r->planes + 20, scaling);
+  face_normal (r->planes     , r->points     , r->points +  3, r->points +  9); vector_set_length (r->planes     , scaling);
+  face_normal (r->planes +  4, r->points + 12, r->points + 15, r->points     ); vector_set_length (r->planes +  4, scaling);
+  face_normal (r->planes +  8, r->points + 9 , r->points +  6, r->points + 21); vector_set_length (r->planes +  8, scaling);
+  face_normal (r->planes + 12, r->points + 12, r->points     , r->points + 21); vector_set_length (r->planes + 12, scaling);
+  face_normal (r->planes + 16, r->points + 3 , r->points + 15, r->points +  6); vector_set_length (r->planes + 16, scaling);
+  face_normal (r->planes + 20, r->points + 15, r->points + 12, r->points + 18); vector_set_length (r->planes + 20, scaling);
   /* re-compute the constants */
   r->planes[ 3] = -(r->planes[ 0] * r->points[ 0] + r->planes[ 1] * r->points[ 1] + r->planes[ 2] * r->points[ 2]);
   r->planes[ 7] = -(r->planes[ 4] * r->points[ 0] + r->planes[ 5] * r->points[ 1] + r->planes[ 6] * r->points[ 2]);
@@ -2005,6 +2045,10 @@ Frustum* frustum_by_matrix (Frustum* r, Frustum* f, GLfloat* m) {
   r->planes[15] = -(r->planes[12] * r->points[ 0] + r->planes[13] * r->points[ 1] + r->planes[14] * r->points[ 2]);
   r->planes[19] = -(r->planes[16] * r->points[ 6] + r->planes[17] * r->points[ 7] + r->planes[18] * r->points[ 8]);
   r->planes[23] = -(r->planes[20] * r->points[12] + r->planes[21] * r->points[13] + r->planes[22] * r->points[14]);
+
+  if (m[16] * m[17] * m[18] < 0.0f) { // left-handed matrix
+    for (i = 0; i < 24; i++) { r->planes[i] = -r->planes[i]; }
+  }
   /* sphere */
 //  point_by_matrix (r->sphere, m);
 //  r->sphere[3] *= scaling;
