@@ -19,6 +19,7 @@
 /**********************************************
  * matrix.h : matrix and other 3D math functions header
  * Copyright (C) 2001-2002 Bertrand 'blam' LAMY
+ * Copyright (C) 2003-2006 Jean-Baptiste 'Jiba' LAMY
  **********************************************/
 
 #define FALSE  0
@@ -79,9 +80,9 @@ GLfloat  point_distance_to           (GLfloat[3], GLfloat[3]);
 double   point_distance_to_d         (double [3], double [3]);
 GLfloat  point_distance_plane        (GLfloat[3], GLfloat[4]);
 void     point_project_on_plane      (GLfloat*  , GLfloat[4]);
-void     point_rotate_lateral        (GLfloat[3], GLfloat);
-void     point_rotate_vertical       (GLfloat[3], GLfloat);
-void     point_rotate_incline        (GLfloat[3], GLfloat);
+void     point_rotate_y              (GLfloat[3], GLfloat);
+void     point_rotate_x              (GLfloat[3], GLfloat);
+void     point_rotate_z              (GLfloat[3], GLfloat);
 void     point_rotate_axe            (GLfloat[3], GLfloat, GLfloat, GLfloat, GLfloat);
 void     point_rotate                (GLfloat[3], GLfloat, GLfloat[3], GLfloat[3]);
 
@@ -113,16 +114,17 @@ void     point4_by_matrix            (GLfloat[4], GLfloat[19]);
 void     vector_by_matrix            (GLfloat[3], GLfloat[19]);
 void     vector_by_matrix_copy       (GLfloat result[3], GLfloat[3], GLfloat[19]);
 GLfloat  length_by_matrix            (GLfloat, GLfloat[19]);
+void     sphere_by_matrix_copy       (GLfloat result[4], GLfloat[4], GLfloat[19]);
 
 void     matrix_set_identity         (GLfloat[19]);
 void     matrix_scale                (GLfloat[19], GLfloat, GLfloat, GLfloat);
-void     matrix_turn_lateral         (GLfloat[19], GLfloat);
-void     matrix_turn_vertical        (GLfloat[19], GLfloat);
-void     matrix_turn_incline         (GLfloat[19], GLfloat);
+void     matrix_turn_y               (GLfloat[19], GLfloat);
+void     matrix_turn_x               (GLfloat[19], GLfloat);
+void     matrix_turn_z               (GLfloat[19], GLfloat);
 void     matrix_turn_axe             (GLfloat[19], GLfloat angle, GLfloat, GLfloat, GLfloat);
-void     matrix_rotate_lateral       (GLfloat[19], GLfloat);
-void     matrix_rotate_vertical      (GLfloat[19], GLfloat);
-void     matrix_rotate_incline       (GLfloat[19], GLfloat);
+void     matrix_rotate_y             (GLfloat[19], GLfloat);
+void     matrix_rotate_x             (GLfloat[19], GLfloat);
+void     matrix_rotate_z             (GLfloat[19], GLfloat);
 void     matrix_rotate_axe           (GLfloat[16], GLfloat angle, GLfloat, GLfloat, GLfloat);
 void     matrix_rotate               (GLfloat[19], GLfloat, GLfloat[3], GLfloat[3]);
 void     matrix_translate            (GLfloat[19], GLfloat, GLfloat, GLfloat);
